/**  
* Title: CrmController.java 
* Description: 
* @author dsc  
* @date 2020年6月5日  
* @version 1.0  
*/
package cn.timer.api.controller.crm;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.StrUtil;
import cn.timer.api.bean.clazz.ClientSourceClass;
import cn.timer.api.bean.clazz.ClientTypeClass;
import cn.timer.api.bean.clazz.IndustryClass;
import cn.timer.api.bean.clazz.SysRegion;
import cn.timer.api.bean.crm.CrmBusinessGroup;
import cn.timer.api.bean.crm.CrmBusinessGroupManager;
import cn.timer.api.bean.crm.CrmBusinessGroupMember;
import cn.timer.api.bean.crm.CrmClientAssociate;
import cn.timer.api.bean.crm.CrmClientContacts;
import cn.timer.api.bean.crm.CrmClientData;
import cn.timer.api.bean.crm.CrmClientFollow;
import cn.timer.api.bean.crm.CrmSeaRule;
import cn.timer.api.bean.yggl.YgglMainEmp;
import cn.timer.api.config.annotation.CurrentUser;
import cn.timer.api.config.annotation.UserBean;
import cn.timer.api.utils.CheckUtil;
import cn.timer.api.utils.Result;
import cn.timer.api.utils.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

/**
 * Title: CrmController.java
 * 
 * @Description:
 * @author dsc
 * @date 2020年6月5日
 * @version 1.0
 */

@RestController
@Api(tags = "6.1客户管理")
@Transactional
@RequestMapping(value = "/crm", produces = { "application/json" })
public class CrmController {

	private Integer getEmpNum(UserBean userBean) {
		return userBean.getEmpNum();
	}

	private Integer getOrgCode(UserBean userBean) {
		return userBean.getOrgCode();
	}

	private YgglMainEmp getYg(Integer orgCode, Integer empNum) {
		return YgglMainEmp.builder().build()
				.selectOne(new QueryWrapper<YgglMainEmp>().lambda().eq(YgglMainEmp::getEmpNum, empNum)
						.eq(YgglMainEmp::getOrgCode, orgCode).select(YgglMainEmp::getName, YgglMainEmp::getHeadUrl));
	}

	private String getEmpName(Integer orgCode, Integer empNum) {
		return getYg(orgCode, empNum).getName();
	}

	@GetMapping("getIndustry")
	@ApiOperation(value = "获取行业列表", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<IndustryClass>> getIndustry(@CurrentUser UserBean userBean) {
		return ResultUtil.data(IndustryClass.builder().build().selectAll());

	}

	@GetMapping("getClientType")
	@ApiOperation(value = "获取客户类型", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<ClientTypeClass>> getClientType(@CurrentUser UserBean userBean) {
		return ResultUtil.data(ClientTypeClass.builder().build().selectAll());

	}

	@GetMapping("getClientSource")
	@ApiOperation(value = "获取客户来源", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<ClientSourceClass>> getClientSource(@CurrentUser UserBean userBean) {
		return ResultUtil.data(ClientSourceClass.builder().build().selectAll());
	}

	@PostMapping("addBusinessGroup")
	@Transactional
	@ApiOperation(value = "新增业务组", httpMethod = "POST", notes = "接口发布说明")
	public Result<Object> addBusinessGroup(@CurrentUser UserBean userBean,
			@RequestBody CrmBusinessGroup crmBusinessGroup) {
		Integer orgCode = getOrgCode(userBean);
		Integer empNum = getEmpNum(userBean);
		CrmBusinessGroup crmBusinessGroupOld = CrmBusinessGroup.builder().build()
				.selectOne(new QueryWrapper<CrmBusinessGroup>().lambda().eq(CrmBusinessGroup::getOrgCode, orgCode)
						.select(CrmBusinessGroup::getSort).last("limit 1").orderByDesc(CrmBusinessGroup::getId));
		if (crmBusinessGroupOld == null)
			crmBusinessGroup.setSort(1);
		else
			crmBusinessGroup.setSort(crmBusinessGroupOld.getSort() + 1);// 排序
//		crmBusinessGroup.setManagerName(getEmpName(orgCode, empNum)); // 设置管理员姓名
		crmBusinessGroup.setCreateUser(empNum); // 创建人
		crmBusinessGroup.setModifyUser(empNum); // 修改人
		crmBusinessGroup.setOrgCode(orgCode);
//		crmBusinessGroup.setMember(member); // ，字符串拼接
		boolean result = crmBusinessGroup.insert();
		if (!result)
			return ResultUtil.success("新增失败");
		Integer gid = crmBusinessGroup.getId();
		Integer[] memberIds = crmBusinessGroup.getMemberIds();
		for (Integer empId : memberIds) {
			CrmBusinessGroupMember.builder().gid(gid).empNum(empId).orgCode(orgCode).build().insert();
		}
		Integer[] managerIds = crmBusinessGroup.getManagerIds();
		for (Integer empId : managerIds) {
			CrmBusinessGroupManager.builder().gid(gid).empNum(empId).orgCode(orgCode).build().insert();
		}

		CrmSeaRule crmSeaRule = crmBusinessGroup.getCrmSeaRules();
		crmSeaRule.setGid(gid);
		crmSeaRule.insert();
		return ResultUtil.success("新增业务组成功");
	}

	@GetMapping("getBusinessGroup")
	@ApiOperation(value = "获取业务组列表", httpMethod = "GET", notes = "接口发布说明")
	public Result<Object> getBusinessGroup(@CurrentUser UserBean userBean) {
		List<CrmBusinessGroup> list = CrmBusinessGroup.builder().build()
				.selectList(new QueryWrapper<CrmBusinessGroup>().lambda()
						.eq(CrmBusinessGroup::getOrgCode, getOrgCode(userBean)).select(CrmBusinessGroup::getId,
								CrmBusinessGroup::getSort, CrmBusinessGroup::getName, CrmBusinessGroup::getModifyTime));
		for (CrmBusinessGroup crmBusinessGroup : list) {
			Integer gid = crmBusinessGroup.getId();
			List<CrmBusinessGroupManager> crmBusinessGroupManagers = CrmBusinessGroupManager.builder().build()
					.selectList(new QueryWrapper<CrmBusinessGroupManager>().lambda()
							.eq(CrmBusinessGroupManager::getGid, gid).select(CrmBusinessGroupManager::getEmpNum));
			List<Map<String, Object>> managers = new ArrayList<Map<String, Object>>();
			for (CrmBusinessGroupManager crmBusinessGroupManager : crmBusinessGroupManagers) {
				Integer empNum = crmBusinessGroupManager.getEmpNum();
				String name = getEmpName(getOrgCode(userBean), empNum);
				Map<String, Object> map = new HashMap<String, Object>();
				map.put("name", name);
				managers.add(map);
			}
			crmBusinessGroup.setManagers(managers);
		}
		return ResultUtil.data(list, "获取成功");
	}

	@GetMapping("getBusinessGroupMembers")
	@ApiOperation(value = "获取业务组列表成员", httpMethod = "GET", notes = "接口发布说明")
	public Result<Object> getBusinessGroupMembers(@CurrentUser UserBean userBean, @RequestParam Integer gid) {
		List<CrmBusinessGroupMember> crmBusinessGroupMembers = CrmBusinessGroupMember.builder().build().selectList(
				new QueryWrapper<CrmBusinessGroupMember>().lambda().eq(CrmBusinessGroupMember::getGid, gid));
		List<YgglMainEmp> ygglMainEmps = new ArrayList<YgglMainEmp>();
		for (CrmBusinessGroupMember crmBusinessGroupMember : crmBusinessGroupMembers) {
			ygglMainEmps.add(getYg(getOrgCode(userBean), crmBusinessGroupMember.getEmpNum()));
		}
		return ResultUtil.data(ygglMainEmps, "获取成功");
	}

	@GetMapping("getBusinessGroupRules") // TODO 扫描规则进行回收操作
	@ApiOperation(value = "获取业务组公海规则", httpMethod = "GET", notes = "接口发布说明")
	public Result<Object> getBusinessGroupRules(@RequestParam Integer gid) {
		List<CrmSeaRule> crmSeaRules = CrmSeaRule.builder().build()
				.selectList(new QueryWrapper<CrmSeaRule>().lambda().eq(CrmSeaRule::getGid, gid));
		return ResultUtil.data(crmSeaRules, "获取成功");
	}

	private SysRegion getSysRegion(Integer keyId) {
		return SysRegion.builder().build()
				.selectOne(new QueryWrapper<SysRegion>().lambda().eq(SysRegion::getKeyId, keyId).select(
						SysRegion::getRegionName, SysRegion::getKeyId, SysRegion::getZipCode,
						SysRegion::getRegionCode));
	}

	@PostMapping("addClient")
	@Transactional
	@ApiOperation(value = "新增客户", httpMethod = "POST", notes = "接口发布说明")
	public Result<Object> addClient(@CurrentUser UserBean userBean, @RequestBody CrmClientData crmClientData) {
		Integer empNum = getEmpNum(userBean);
		Integer orgCode = getOrgCode(userBean);
		crmClientData.setCreateUser(empNum);
		SysRegion district = getSysRegion(crmClientData.getKeyId());
//		crmClientData.setZipCode(district.getZipCode());
		crmClientData.setDistrict(district.getRegionName());
		SysRegion city = getSysRegion(district.getRegionCode());
		crmClientData.setCity(city.getRegionName());
		SysRegion province = getSysRegion(city.getRegionCode());
		crmClientData.setProvince(province.getRegionName());
		crmClientData.setBelongUserName(getEmpName(orgCode, crmClientData.getBelongUser()));
		crmClientData.setOrgCode(getOrgCode(userBean));
		Integer belongGroup = crmClientData.getBelongGroup();
		String belongGroupName = CrmBusinessGroup.builder().id(belongGroup).build().selectById().getName();
		crmClientData.setBelongGroupName(belongGroupName);

		String clientSourceName = ClientSourceClass.builder().id(crmClientData.getClientSource()).build().selectById()
				.getName(); // 客户来源
		String clientTypeName = ClientTypeClass.builder().id(crmClientData.getClientType()).build().selectById()
				.getName(); // 客户类型
		String industryName = IndustryClass.builder().id(crmClientData.getIndustry()).build().selectById().getName();// 行业名

		crmClientData.setClientSourceName(clientSourceName);
		crmClientData.setClientTypeName(clientTypeName);
		crmClientData.setIndustryName(industryName);
		if (!crmClientData.insert())
			return ResultUtil.error("新增失败");
		Integer crmDataId = crmClientData.getId();
		Integer[] ids = crmClientData.getIds();
		for (Integer id : ids) {
			CrmClientAssociate.builder().cid(crmDataId).associateId(id).orgCode(getOrgCode(userBean)).build().insert();
		}
		List<CrmClientContacts> crmClientContacts = crmClientData.getCrmClientContacts();
		for (CrmClientContacts crmClientContact : crmClientContacts) {
			crmClientContact.setCid(crmDataId);
			crmClientContact.insert();
		}
		crmClientData.setContactsNum(crmClientContacts.size());
		crmClientData.updateById();
		return ResultUtil.success("新增成功");
	}

	@PostMapping("editClient")
	@Transactional
	@ApiOperation(value = "编辑客户", httpMethod = "POST", notes = "接口发布说明")
	public Result<Object> editClient(@CurrentUser UserBean userBean, @RequestBody CrmClientData crmClientData) {
		Integer cid = crmClientData.getId();
		if (cid == null)
			return ResultUtil.error("请传入id再试");
		CrmClientData oldData = CrmClientData.builder().id(cid).build().selectById();
		if (oldData == null)
			return ResultUtil.error("请确认该客户数据是否存在");
		if (oldData.getClientStatus() == 5)
			return ResultUtil.error("请解封后继续编辑操作");
		List<CrmClientContacts> crmClientContacts = crmClientData.getCrmClientContacts();
		for (CrmClientContacts crmClientContact : crmClientContacts) {
			crmClientContact.updateById();
		}
		List<CrmClientAssociate> oldAssociates = CrmClientAssociate.builder().build()
				.selectList(new QueryWrapper<CrmClientAssociate>().lambda().eq(CrmClientAssociate::getCid, cid));
		List<Integer> oldIdsList = oldAssociates.stream().map(CrmClientAssociate::getAssociateId)
				.collect(Collectors.toList()); // 旧协助人list
		Integer[] ids = crmClientData.getIds();
		List<Integer> newIdsList = CollUtil.newArrayList(ids); // 新协助人list
		List<Integer> delList = CheckUtil.reduce(oldIdsList, newIdsList); // 要删的协助人
		for (Integer id : delList) {
			CrmClientAssociate.builder().build().delete(new QueryWrapper<CrmClientAssociate>().lambda()
					.eq(CrmClientAssociate::getCid, cid).eq(CrmClientAssociate::getAssociateId, id));
		}
		List<Integer> addList = CheckUtil.reduce(newIdsList, oldIdsList); // 要加的协助人
		for (Integer id : addList) {
			CrmClientAssociate.builder().cid(cid).associateId(id).build().insert();
		}
		crmClientData.setModifyUser(getEmpNum(userBean));
		if (crmClientData.updateById())
			return ResultUtil.success("编辑成功");
		return ResultUtil.error("编辑失败");
	}

	@GetMapping("getClientAssociate")
	@ApiOperation(value = "获取协作人列表", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<Map<String, Object>>> getClientAssociate(@CurrentUser UserBean userBean,
			@RequestParam Integer cid) {
		List<CrmClientAssociate> crmClientAssociates = CrmClientAssociate.builder().build()
				.selectList(new QueryWrapper<CrmClientAssociate>().lambda().eq(CrmClientAssociate::getCid, cid));
		List<Map<String, Object>> list = new ArrayList<Map<String, Object>>();
		for (CrmClientAssociate crmClientAssociate : crmClientAssociates) {
			Integer orgCode = crmClientAssociate.getOrgCode();
			Integer empNum = crmClientAssociate.getAssociateId();
			YgglMainEmp ygglMainEmp = getYg(orgCode, empNum);
			String name = ygglMainEmp.getName();
			String headUrl = ygglMainEmp.getHeadUrl();
			Map<String, Object> map = new HashMap<String, Object>();
			map.put("name", name);
			map.put("empNum", empNum);
			map.put("headUrl", headUrl);
			list.add(map);
		}
		return ResultUtil.data(list, "获取成功");
	}

	// 归属我的客户
	private List<CrmClientData> getMyCilent(Integer empNum, Integer orgCode, Integer groupId, Integer status,
			String like) {
		return CrmClientData.builder().build()
				.selectList(new QueryWrapper<CrmClientData>().lambda().eq(CrmClientData::getOrgCode, orgCode)
						.eq(CrmClientData::getBelongUser, empNum)
						.eq(groupId != null && groupId >= 0, CrmClientData::getBelongGroup, groupId)
						.eq(status != null && status >= 0, CrmClientData::getClientStatus, status)
						.like(!StrUtil.isBlank(like), CrmClientData::getClientName, like).or()
						.like(!StrUtil.isBlank(like), CrmClientData::getClientCellphone, like)
						.orderByDesc(CrmClientData::getCreateTime));
	}

	// 我协作的客户
	private List<CrmClientData> getMyAssociateCilent(Integer empNum, Integer orgCode, Integer groupId, Integer status,
			String like) {
		List<CrmClientAssociate> crmClientAssociates = CrmClientAssociate.builder().build().selectList(
				new QueryWrapper<CrmClientAssociate>().lambda().eq(CrmClientAssociate::getAssociateId, empNum)
						.eq(CrmClientAssociate::getOrgCode, orgCode).select(CrmClientAssociate::getCid));
		List<CrmClientData> myCrmClientAssociates = new ArrayList<CrmClientData>();
		for (CrmClientAssociate crmClientAssociate : crmClientAssociates) {
			Integer cid = crmClientAssociate.getCid();
			CrmClientData crmClientData = CrmClientData.builder().build()
					.selectOne(new QueryWrapper<CrmClientData>().lambda().eq(CrmClientData::getId, cid)
							.eq(groupId != null && groupId >= 0, CrmClientData::getBelongGroup, groupId)
							.eq(status != null && status >= 0, CrmClientData::getClientStatus, status)
							.like(!StrUtil.isBlank(like), CrmClientData::getClientName, like).or()
							.like(!StrUtil.isBlank(like), CrmClientData::getClientCellphone, like));
			myCrmClientAssociates.add(crmClientData);
		}
		return myCrmClientAssociates;
	}

	// 我的全部客户
	private List<CrmClientData> getMyTotalCilent(Integer empNum, Integer orgCode, Integer groupId, Integer status,
			String like) {
		return CheckUtil.distinct(getMyCilent(empNum, orgCode, groupId, status, like),
				getMyAssociateCilent(empNum, orgCode, groupId, status, like), true);
	}

	// 我关注的客户
	private List<CrmClientData> getMyStarCilent(Integer empNum, Integer orgCode, Integer groupId, Integer status) {
		return getMyTotalCilent(empNum, orgCode, groupId, status, null).stream()
				.filter(item -> item.getStar().equals(1)).collect(Collectors.toList());
	}

	// 七天未跟进的客户
	private List<CrmClientData> getCilentFollowOvertime(Integer empNum, Integer orgCode, Integer groupId,
			Integer status) {
		return getMyTotalCilent(empNum, orgCode, groupId, status, null).stream()
				.filter(item -> item.getLastFollowTime() != null
						&& item.getLastFollowTime().before(DateUtil.offsetDay(new Date(), -7)))
				.collect(Collectors.toList());
	}

	@GetMapping("getMyCilentTotal")
	@ApiOperation(value = "获取我的客户列表(全)", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<CrmClientData>> getMyCilentTotal(@CurrentUser UserBean userBean,
			@RequestParam(required = false, defaultValue = "0") Integer type,
			@RequestParam(required = false) Integer groupId, @RequestParam(required = false) Integer status,
			@RequestParam(required = false) String like) {
		Integer empNum = getEmpNum(userBean);
		Integer orgCode = getOrgCode(userBean);
		List<CrmClientData> crmClientDatas = null;
		if (type == null || type == 0) // 全部客户(去重)
			crmClientDatas = CheckUtil.distinct(getMyCilent(empNum, orgCode, groupId, status, like),
					getMyAssociateCilent(empNum, orgCode, groupId, status, like), true);
		else if (type == 1) // 我负责的客户
			crmClientDatas = getMyCilent(empNum, orgCode, groupId, status, like);
		else if (type == 2) // 我协作的客户
			crmClientDatas = getMyAssociateCilent(empNum, orgCode, groupId, status, like);
		else if (type == 3) // 我关注的客户
			crmClientDatas = getMyStarCilent(empNum, orgCode, groupId, status);
		else if (type == 4) // 七天未跟进的客户
			crmClientDatas = getCilentFollowOvertime(empNum, orgCode, groupId, status);
		else
			return ResultUtil.error("查询失败");
		Collections.sort(crmClientDatas, Comparator.comparing(CrmClientData::getCreateTime).reversed()); // 按时间降序排序
		return ResultUtil.data(crmClientDatas, "查询成功");
	}

	@GetMapping("getSeaCilentTotal")
	@ApiOperation(value = "获取公海客户列表(全)", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<CrmClientData>> getSeaCilentTotal(@CurrentUser UserBean userBean,
			@RequestParam(required = false) Integer type, @RequestParam(required = false) Integer groupId,
			@RequestParam(required = false) Integer status, @RequestParam(required = false) String like) {
		Integer empNum = getEmpNum(userBean);
		Integer orgCode = getOrgCode(userBean);
		// 当前用户加入的组
		List<CrmBusinessGroupMember> members = CrmBusinessGroupMember.builder().build()
				.selectList(new QueryWrapper<CrmBusinessGroupMember>().lambda()
						.eq(CrmBusinessGroupMember::getEmpNum, empNum).eq(CrmBusinessGroupMember::getOrgCode, orgCode));
		List<Integer> gids1 = members.stream().map(CrmBusinessGroupMember::getGid).collect(Collectors.toList());
		// 当前用户管理的组
		List<CrmBusinessGroupManager> managers = CrmBusinessGroupManager.builder().build()
				.selectList(new QueryWrapper<CrmBusinessGroupManager>().lambda()
						.eq(CrmBusinessGroupManager::getEmpNum, empNum)
						.eq(CrmBusinessGroupManager::getOrgCode, orgCode));
		List<Integer> gids2 = managers.stream().map(CrmBusinessGroupManager::getGid).collect(Collectors.toList());

		List<Integer> myGids = CheckUtil.distinct(gids1, gids2, true); // 当前用户所有业务组id
		List<CrmClientData> crmClientDatas = new ArrayList<CrmClientData>();
		for (Integer gid : myGids) {
			crmClientDatas.add(CrmClientData.builder().build()
					.selectOne(new QueryWrapper<CrmClientData>().lambda().eq(CrmClientData::getBelongGroup, gid)
							.eq(CrmClientData::getBelongUser, 0)
							.eq(type != null && type >= 0, CrmClientData::getClientType, type)
							.eq(groupId != null && groupId >= 0, CrmClientData::getBelongGroup, groupId)
							.eq(status != null && status >= 0, CrmClientData::getClientStatus, status)
							.like(!StrUtil.isBlank(like), CrmClientData::getClientName, like).or()
							.like(!StrUtil.isBlank(like), CrmClientData::getClientCellphone, like)));
		} // TODO
		return ResultUtil.data(crmClientDatas, "查询成功");
	}

	@PostMapping("addCilentLinkMan")
	@Transactional
	@ApiOperation(value = "新增/编辑客户联系人", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> addCilentLinkMan(@CurrentUser UserBean userBean, CrmClientContacts crmClientContacts) {
		if (crmClientContacts.getId() == null) {
			Integer num = CrmClientData.builder().build().selectById().getContactsNum();
			CrmClientData.builder().id(crmClientContacts.getCid()).contactsNum(++num).build().updateById();
		}
		if (crmClientContacts.insertOrUpdate())
			return ResultUtil.success("新增/编辑成功");
		return ResultUtil.success("新增/编辑失败");

	}

	@DeleteMapping("delCilentLinkMan")
	@Transactional
	@ApiOperation(value = "删除客户联系人", httpMethod = "DELETE", notes = "接口发布说明")
	public Result<Void> delCilentLinkMan(@CurrentUser UserBean userBean, CrmClientContacts crmClientContacts) {
		Integer num = CrmClientData.builder().build().selectById().getContactsNum();
		CrmClientData.builder().id(crmClientContacts.getCid()).contactsNum(--num).build().updateById();
		if (crmClientContacts.deleteById())
			return ResultUtil.success("删除成功");
		return ResultUtil.success("删除失败");

	}

	@GetMapping("getCilentLinkMan")
	@ApiOperation(value = "获取客户联系人列表", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<CrmClientContacts>> getCilentLinkMan(@CurrentUser UserBean userBean, Integer cid) {
		return ResultUtil.data(CrmClientContacts.builder().build()
				.selectList(new QueryWrapper<CrmClientContacts>().lambda().eq(CrmClientContacts::getCid, cid)), "获取成功");

	}

	@PostMapping("addOrCancelClientStar")
	@ApiOperation(value = "新增客户收藏/取消收藏(仅归属人)", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> addOrCancelClientStar(@CurrentUser UserBean userBean, @RequestParam Integer cid) {
		Integer empNum = getEmpNum(userBean);
		CrmClientData crmClientData = CrmClientData.builder().id(cid).build().selectById();
		if (crmClientData == null)
			return ResultUtil.error("请确认该客户是否存在");
		if (!crmClientData.getBelongUser().equals(empNum))
			return ResultUtil.error("不是当前客户归属人");
		Integer star = crmClientData.getStar();
		if (star == null || star == 0) {
			crmClientData.setStar(1);
			crmClientData.updateById();
			return ResultUtil.success("收藏成功");
		} else if (star == 1) {
			crmClientData.setStar(0);
			crmClientData.updateById();
			return ResultUtil.success("取消收藏成功");
		} else
			return ResultUtil.error("失败");
	}

	@PostMapping("addClientFollow")
	@Transactional
	@ApiOperation(value = "新增跟进记录", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> addClientFollow(@CurrentUser UserBean userBean, @RequestBody CrmClientFollow crmClientFollow) {

		Integer cid = crmClientFollow.getCid();
		if (cid == null)
			return ResultUtil.error("请传入id再试");
		CrmClientData oldData = CrmClientData.builder().id(cid).build().selectById();
		if (oldData == null)
			return ResultUtil.error("请确认该客户数据是否存在");
		if (oldData.getClientStatus() == 5)
			return ResultUtil.error("请解封后继续编辑操作");

		Integer orgCode = getOrgCode(userBean);
		Integer empNum = getEmpNum(userBean);

		CrmClientData.builder().id(crmClientFollow.getCid()).lastFollowTime(new Date()).build().updateById(); // 更新客户最近跟进时间

		crmClientFollow.setOrgCode(orgCode);
		crmClientFollow.setCreateUserName(getEmpName(orgCode, empNum));

		if (crmClientFollow.insert())
			return ResultUtil.success("新增成功");
		return ResultUtil.error("新增失败");
	}

	@GetMapping("getCilentFollow")
	@ApiOperation(value = "获取跟进记录", httpMethod = "GET", notes = "接口发布说明")
	public Result<List<CrmClientFollow>> getCilentFollow(@CurrentUser UserBean userBean, @RequestParam Integer cid) {
		Integer orgCode = getOrgCode(userBean);
		return ResultUtil.data(CrmClientFollow.builder().build().selectList(new QueryWrapper<CrmClientFollow>().lambda()
				.eq(CrmClientFollow::getOrgCode, orgCode).eq(CrmClientFollow::getCid, cid)), "获取成功");
	}

	@PostMapping("addClientArchive")
	@ApiOperation(value = "客户封档/解封", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> addClientArchive(@CurrentUser UserBean userBean, @RequestParam Integer cid) {
		Integer empNum = getEmpNum(userBean);
		CrmClientData crmClientData = CrmClientData.builder().id(cid).build().selectById();
		if (crmClientData == null)
			return ResultUtil.error("请确认该客户是否存在");
		if (!crmClientData.getBelongUser().equals(empNum))
			return ResultUtil.error("不是当前客户归属人");
		Integer status = crmClientData.getClientStatus();
		if (status != null && status != 5) {
			crmClientData.setClientStatus(5);
			crmClientData.updateById();
			return ResultUtil.success("封档成功");
		} else if (status == 5) {
			crmClientData.setClientStatus(0); // 解封后默认无状态
			crmClientData.updateById();
			return ResultUtil.success("解封成功");
		} else
			return ResultUtil.error("失败");
	}

	@PostMapping("designateClientBelonger")
	@ApiOperation(value = "指派客户归属人", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> designateClientBelonger(@CurrentUser UserBean userBean, @RequestParam Integer cid,
			@RequestParam Integer belongUser) {
		if (cid == null)
			return ResultUtil.error("请传入id再试");
		CrmClientData oldData = CrmClientData.builder().id(cid).build().selectById();
		if (oldData == null)
			return ResultUtil.error("请确认该客户数据是否存在");
		if (!oldData.getBelongUser().equals(getEmpNum(userBean)))
			return ResultUtil.error("不是当前客户归属人");
		if (oldData.getClientStatus() == 5)
			return ResultUtil.error("请解封后继续编辑操作");

		Integer belongGroup = oldData.getBelongGroup();
		CrmBusinessGroupMember crmBusinessGroupMember = CrmBusinessGroupMember.builder().build()
				.selectOne(new QueryWrapper<CrmBusinessGroupMember>().lambda()
						.eq(CrmBusinessGroupMember::getGid, belongGroup)
						.eq(CrmBusinessGroupMember::getEmpNum, belongUser));
		if (crmBusinessGroupMember == null)
			return ResultUtil.error("指派失败,该用户不是本组成员");

		if (CrmClientData.builder().id(cid).belongUser(belongUser)
				.belongUserName(getEmpName(getOrgCode(userBean), belongUser)).build().updateById())
			return ResultUtil.success("指派成功");
		return ResultUtil.error("指派失败");

	}

	@PostMapping("freeClientToSea")
	@Transactional
	@ApiOperation(value = "释放客户到公海", httpMethod = "POST", notes = "接口发布说明")
	public Result<Void> freeClientToSea(@CurrentUser UserBean userBean, @RequestParam Integer cid) {
		if (cid == null)
			return ResultUtil.error("请传入id再试");
		CrmClientData oldData = CrmClientData.builder().id(cid).build().selectById();
		if (oldData == null)
			return ResultUtil.error("请确认该客户数据是否存在");
		if (!oldData.getBelongUser().equals(getEmpNum(userBean)))
			return ResultUtil.error("不是当前客户归属人");
		if (oldData.getClientStatus() == 5)
			return ResultUtil.error("请解封后继续编辑操作");
		oldData.setBelongUser(0); // 0-进入客户公海
		oldData.setBelongUserName(null);
		if (oldData.updateById() && CrmClientAssociate.builder().build()
				.delete(new QueryWrapper<CrmClientAssociate>().lambda().eq(CrmClientAssociate::getCid, cid)))
			return ResultUtil.success("释放成功");
		return ResultUtil.error("释放失败");
	}

	// TODO

}
